@@ -14,6 +14,8 @@ __all__ = [
     "SearchResponse",
     "PlaylistResponse",
     "HealthResponse",
+    "DownloadFileProgress",
+    "SetupProgress"
 ]
 
 
@@ -22,10 +24,10 @@ class VideoInfo(BaseModel):
 
     url: str
     title: str
-    duration: int = Field(0, ge=0)
+    duration: float = Field(0, ge=-1)
     uploader: str
-    view_count: int = Field(0, ge=0)
-    like_count: Optional[int] = Field(None, ge=0)
+    view_count: int = Field(0, ge=-1)
+    like_count: Optional[int] = Field(None, ge=-1)
     description: str = ""
     thumbnail: str = ""
     upload_date: str = ""
@@ -85,6 +87,7 @@ class DownloadConfig(BaseModel):
     subtitle_lang: str = Field(default="en", description="Subtitle language code")
     write_thumbnail: bool = Field(default=False, description="Download thumbnail")
     embed_thumbnail: bool = Field(default=False, description="Embed thumbnail")
+    embed_metadata: bool = Field(default=True, description="Embed metadata")
     write_info_json: bool = Field(default=False, description="Write info JSON file")
     custom_filename: Optional[str] = Field(
         default=None, description="Custom filename template"
@@ -152,6 +155,31 @@ class DownloadProgress(BaseModel):
     class Config:
         json_encoders = {float: lambda v: round(v, 2)}
 
+class DownloadFileProgress(BaseModel):
+    """Progress information for File downloads"""
+
+    status: str = "downloading"
+    downloaded_bytes: int = 0
+    total_bytes: int = 0
+    percentage: float = Field(0.0, ge=0.0, le=100.0)
+
+    @property
+    def is_complete(self) -> bool:
+        return self.status == "finished"
+
+    class Config:
+        json_encoders = {float: lambda v: round(v, 2)}
+
+class SetupProgress(BaseModel):
+    """Progress information for File downloads"""
+
+    file: str = "yt-dlp"
+    download_file_progress: DownloadFileProgress= Field(description="the progress of the file being downloaded")
+
+
+    class Config:
+        json_encoders = {float: lambda v: round(v, 2)}
+
 
 # API Response Models
 class DownloadRequest(BaseModel):
