@@ -1,5 +1,5 @@
 """
-AsyncYT - A comprehensive async YouTube downloader library
+AsyncYT - A comprehensive async Any website downloader library
 Uses yt-dlp and ffmpeg with automatic binary management
 """
 
@@ -7,16 +7,19 @@ import asyncio
 import json
 import os
 import platform
+import re
 import shutil
 import zipfile
 from pathlib import Path
-from typing import Any, Awaitable, Callable, Dict, List, Optional, Union
+from typing import Any, AsyncGenerator, Awaitable, Callable, Dict, List, Optional, Union
 import aiofiles
 import aiohttp
 import logging
 
 from .enums import AudioFormat, VideoFormat, Quality
 from .basemodels import (
+    DownloadFileProgress,
+    SetupProgress,
     VideoInfo,
     DownloadConfig,
     DownloadProgress,
@@ -28,7 +31,7 @@ from .basemodels import (
     PlaylistResponse,
     HealthResponse,
 )
-from .utils import call_callback
+from .utils import call_callback, get_unique_filename
 
 logger = logging.getLogger(__name__)
 
@@ -41,73 +44,85 @@ class Downloader:
     def __init__(self):
         self.project_root = Path.cwd()
         self.bin_dir = self.project_root / "bin"
-        self.ytdlp_path = None
-        self.ffmpeg_path = None
-        self.ffprobe_path = None
+        system = platform.system().lower()
+
+        self.ytdlp_path = (
+            self.bin_dir / "yt-dlp.exe"
+            if system == "windows"
+            else self.bin_dir / "yt-dlp"
+        )
+        self.ffmpeg_path = (
+            self.bin_dir / "ffmpeg.exe" if system == "windows" else "ffmpeg"
+        )
+
+    async def setup_binaries_generator(self) -> AsyncGenerator[SetupProgress, Any]:
+        """Download and setup yt-dlp and ffmpeg binaries with yield SetupProgress"""
+        self.bin_dir.mkdir(exist_ok=True)
+
+        # Setup yt-dlp
+        async for progress in self._setup_ytdlp():
+            yield progress
+
+        # Setup ffmpeg
+        async for progress in self._setup_ffmpeg():
+            yield progress
+
+        logger.info("All binaries are ready!")
 
     async def setup_binaries(self) -> None:
         """Download and setup yt-dlp and ffmpeg binaries"""
         self.bin_dir.mkdir(exist_ok=True)
 
         # Setup yt-dlp
-        await self._setup_ytdlp()
+        async for _ in self._setup_ytdlp():
+            pass
 
         # Setup ffmpeg
-        await self._setup_ffmpeg()
+        async for _ in self._setup_ffmpeg():
+            pass
 
         logger.info("All binaries are ready!")
 
-    async def _setup_ytdlp(self) -> None:
+    async def _setup_ytdlp(self) -> AsyncGenerator[SetupProgress, Any]:
         """Download yt-dlp binary"""
         system = platform.system().lower()
 
         if system == "windows":
-            filename = "yt-dlp.exe"
             url = "https://github.com/yt-dlp/yt-dlp/releases/latest/download/yt-dlp.exe"
         else:
-            filename = "yt-dlp"
             url = "https://github.com/yt-dlp/yt-dlp/releases/latest/download/yt-dlp"
 
-        self.ytdlp_path = self.bin_dir / filename
-
         if not self.ytdlp_path.exists():
             logger.info(f"Downloading yt-dlp...")
-            await self._download_file(url, self.ytdlp_path)
+            async for progress in self._download_file(url, self.ytdlp_path):
+                yield SetupProgress(file="yt-dlp", download_file_progress=progress)
 
             if system != "windows":
                 os.chmod(self.ytdlp_path, 0o755)
 
-    async def _setup_ffmpeg(self) -> None:
+    async def _setup_ffmpeg(self) -> AsyncGenerator[SetupProgress, Any]:
         """Download ffmpeg binary"""
         system = platform.system().lower()
 
         if system == "windows":
             self.ffmpeg_path = self.bin_dir / "ffmpeg.exe"
-            self.ffprobe_path = self.bin_dir / "ffprobe.exe"
 
             if not self.ffmpeg_path.exists():
                 logger.info(f"Downloading ffmpeg for Windows...")
                 url = "https://github.com/BtbN/FFmpeg-Builds/releases/latest/download/ffmpeg-n7.1-latest-win64-lgpl-7.1.zip"
                 temp_file = self.bin_dir / "ffmpeg.zip"
 
-                await self._download_file(url, temp_file)
+                async for progress in self._download_file(url, temp_file):
+                    yield SetupProgress(file="ffmpeg", download_file_progress=progress)
+                progress.status = "extracting"
+                yield SetupProgress(file="ffmpeg", download_file_progress=progress)
                 await self._extract_ffmpeg_windows(temp_file)
                 temp_file.unlink()
 
-        elif system == "darwin":  # macOS
+        else:
             # For macOS, we'll check if ffmpeg is available via Homebrew
             if shutil.which("ffmpeg"):
                 self.ffmpeg_path = "ffmpeg"
-                self.ffprobe_path = "ffprobe"
-            else:
-                logger.warning(
-                    "ffmpeg not found. Please install via: brew install ffmpeg"
-                )
-
-        else:  # Linux
-            if shutil.which("ffmpeg"):
-                self.ffmpeg_path = "ffmpeg"
-                self.ffprobe_path = "ffprobe"
             else:
                 logger.warning(
                     "ffmpeg not found. Please install via your package manager"
@@ -122,16 +137,70 @@ class Downloader:
                     file_info.filename = os.path.basename(file_info.filename)
                     zip_ref.extract(file_info, self.bin_dir)
 
-    async def _download_file(self, url: str, filepath: Path) -> None:
-        """Download a file asynchronously"""
-        async with aiohttp.ClientSession() as session:
-            async with session.get(url) as response:
-                if response.status == 200:
-                    async with aiofiles.open(filepath, "wb") as f:
-                        async for chunk in response.content.iter_chunked(8192):
-                            await f.write(chunk)
-                else:
-                    raise Exception(f"Failed to download {url}: {response.status}")
+    async def _download_file(
+        self, url: str, filepath: Path, max_retries: int = 5
+    ) -> AsyncGenerator[DownloadFileProgress, Any]:
+        """Download a file asynchronously with retries, timeout, resume support, and file size verification"""
+        temp_filepath = filepath.with_suffix(filepath.suffix + ".part")
+        attempt = 0
+        backoff = 2
+        while attempt < max_retries:
+            try:
+                resume_pos = 0
+                if temp_filepath.exists():
+                    resume_pos = temp_filepath.stat().st_size
+                headers = {}
+                if resume_pos > 0:
+                    headers["Range"] = f"bytes={resume_pos}-"
+
+                timeout_obj = aiohttp.ClientTimeout(
+                    total=None, sock_connect=30, sock_read=60
+                )
+                async with aiohttp.ClientSession(timeout=timeout_obj) as session:
+                    async with session.get(url, headers=headers) as response:
+                        if response.status in (200, 206):
+                            mode = (
+                                "ab"
+                                if resume_pos > 0 and response.status == 206
+                                else "wb"
+                            )
+                            async with aiofiles.open(temp_filepath, mode) as f:
+                                downloaded = resume_pos
+                                total = (
+                                    int(response.headers.get("Content-Length", 0))
+                                    + resume_pos
+                                    if response.status == 206
+                                    else int(response.headers.get("Content-Length", 0))
+                                )
+                                async for chunk in response.content.iter_chunked(8192):
+                                    await f.write(chunk)
+                                    downloaded += len(chunk)
+                                    percent = (downloaded / total) * 100
+                                    yield DownloadFileProgress(
+                                        status="downloading",
+                                        downloaded_bytes=downloaded,
+                                        total_bytes=total,
+                                        percentage=percent,
+                                    )
+                            # Verify file size
+                            if total > 0 and temp_filepath.stat().st_size != total:
+                                raise Exception(
+                                    f"Incomplete download for {filepath.name}: expected {total}, got {temp_filepath.stat().st_size}"
+                                )
+                            temp_filepath.rename(filepath)
+                            return
+                        else:
+                            raise Exception(
+                                f"Failed to download {url}: {response.status}"
+                            )
+            except Exception as e:
+                attempt += 1
+                wait = backoff**attempt
+                logger.warning(
+                    f"Download attempt {attempt} failed for {url}: {e}. Retrying in {wait}s..."
+                )
+                await asyncio.sleep(wait)
+        raise Exception(f"Failed to download {url} after {max_retries} attempts.")
 
     async def get_video_info(self, url: str) -> VideoInfo:
         """Get video information without downloading"""
@@ -202,49 +271,54 @@ class Downloader:
             cwd=output_dir,
         )
 
-        output_file: str
+        output_file: Optional[str] = None
 
+        output = []
         # Monitor progress
         async for line in self._read_process_output(process):
             line = line.strip()
+            output.append(line)
 
             if line:
                 old_percentage = progress.percentage
                 self._parse_progress(line, progress)
 
-                if progress_callback and (
-                    progress.percentage != old_percentage
-                    or progress.downloaded_bytes > 0
-                    or progress.total_bytes > 0
-                ):
+                if progress_callback and progress.percentage > old_percentage:
                     await call_callback(progress_callback, progress)
 
-                # Capture output filename
-                if "[download] Destination:" in line:
-                    output_file = line.split("Destination: ")[1]
-                elif "[download]" in line and "has already been downloaded" in line:
-                    output_file = line.split()[1]
+                # Robust output filename extraction: only set if line is an absolute path, has a valid extension, and file exists
+                valid_exts = (
+                    ".mp3",
+                    ".m4a",
+                    ".wav",
+                    ".flac",
+                    ".ogg",
+                    ".wav",
+                    ".mp4",
+                    ".webm",
+                    ".mkv",
+                    ".avi",
+                    ".opus",
+                    ".aac",
+                )
+                if not output_file and line.lower().endswith(valid_exts):
+                    output_file = line
 
-        await process.wait()
+        returncode = await process.wait()
 
-        if process.returncode != 0:
-            raise Exception(f"Download failed for {url}")
+        if returncode != 0:
+            raise RuntimeError(
+                f"Download failed for {url}\n"
+                f"CMD: {" ".join(cmd)}\n"
+                f"Return code: {returncode}\n"
+                f"Output:\n" + "\n".join(output)
+            )
 
         if progress_callback:
             progress.status = "finished"
             progress.percentage = 100.0
             await call_callback(progress_callback, progress)
-        if config and (config.video_format or config.audio_format):
-            # Support both Enum and string for video/audio format
-            if config.video_format:
-                ext = config.video_format.value if hasattr(config.video_format, "value") else str(config.video_format)
-            elif config.audio_format:
-                ext = config.audio_format.value if hasattr(config.audio_format, "value") else str(config.audio_format)
-            else:
-                return output_file
-            base, _ = os.path.splitext(output_file)
-            output_file = f"{base}.{ext}"
-        return output_file
+        return output_file  # type: ignore
 
     async def health_check(self) -> HealthResponse:
         """Check if all binaries are available and working"""
@@ -325,10 +399,15 @@ class Downloader:
             # Download the video
             filename = await self.download(request.url, config, progress_callback)
 
+            file = Path(filename)
+            title = re.sub(r'[\\/:"*?<>|]', "_", video_info.title)
+            new_file = get_unique_filename(file, title)
+            file = file.rename(new_file)
+
             return DownloadResponse(
                 success=True,
                 message="Download completed successfully",
-                filename=filename,
+                filename=str(file.absolute()),
                 video_info=video_info,
             )
 
@@ -482,18 +561,15 @@ class Downloader:
 
         # Basic options
         cmd.extend(["--no-warnings", "--progress"])
-
         # Quality selection
         if config.extract_audio:
+            audio = AudioFormat(config.audio_format).value if config.audio_format else "best"
+            audio = "vorbis" if audio == "ogg" else audio # Somehow it's not ogg but called vorbis??
             cmd.extend(
                 [
                     "-x",
                     "--audio-format",
-                    (
-                        AudioFormat(config.audio_format).value
-                        if config.audio_format
-                        else "mp3"
-                    ),
+                    audio,
                 ]
             )
         else:
@@ -547,6 +623,8 @@ class Downloader:
             cmd.extend(["--proxy", config.proxy])
         if config.rate_limit:
             cmd.extend(["--limit-rate", config.rate_limit])
+        if config.embed_metadata:
+            cmd.append("--add-metadata")
 
         # Retry options
         cmd.extend(["--retries", str(config.retries)])
@@ -570,18 +648,23 @@ class Downloader:
                 "download:PROGRESS|%(progress._percent_str)s|%(progress._downloaded_bytes_str)s|%(progress._total_bytes_str)s|%(progress._speed_str)s|%(progress._eta_str)s",
             ]
         )
+        cmd.append("--no-update")
         cmd.append("--newline")
+        cmd.append("--restrict-filenames")
+        cmd.extend(["--print", "after_move:filepath"])
+
         cmd.append(url)
-        logger.debug(cmd)
         return cmd
 
     async def _read_process_output(self, process):
-        """Read process output line by line"""
+        """Read process output line by line as UTF-8 (with replacement for bad chars)"""
+        assert process.stdout is not None, "Process must have stdout=PIPE"
+
         while True:
             line = await process.stdout.readline()
             if not line:
                 break
-            yield line.decode("utf-8", errors="ignore")
+            yield line.decode("utf-8", errors="replace").rstrip()
 
     def _parse_progress(self, line: str, progress: DownloadProgress) -> None:
         """Parse progress information from yt-dlp output"""
